@@ -14,6 +14,10 @@
 ##'   for point-wise intervals, or `"simultaneous"` for simultaneous intervals.
 ##' @param nsim integer; the number of simulations used in computing the
 ##'   simultaneous intervals.
+##' @param ncores number of cores for generating random variables from a
+##'   multivariate normal distribution. Passed to [mvnfast::rmvn()].
+##'   Parallelization will take place only if OpenMP is supported (but appears
+##'   to work on Windows with current `R`).
 ##' @param ... additional arguments for methods
 ##'
 ##' @return a data frame with components:
@@ -27,12 +31,12 @@
 ##' @export
 ##'
 ##' @examples
-##' suppressPackageStartupMessages(library("mgcv"))
+##' load_mgcv()
 ##' \dontshow{
 ##' set.seed(2)
-##' op <- options(digits = 5)
+##' op <- options(digits = 3, cli.unicode = FALSE)
 ##' }
-##' dat <- gamSim(1, n = 400, dist = "normal", scale = 2)
+##' dat <- gamSim(1, n = 1000, dist = "normal", scale = 2)
 ##' mod <- gam(y ~ s(x0) + s(x1) + s(x2) + s(x3), data = dat, method = "REML")
 ##'
 ##' ## first derivatives of all smooths...
@@ -43,12 +47,15 @@
 ##' head(ci)
 ##'
 ##' ## simultaneous interval for smooth term of x1
+##' \dontshow{
 ##' set.seed(42)
-##' x1.sint <- confint(fd, parm = "x1", type = "simultaneous", nsim = 1000)
+##' }
+##' x1.sint <- confint(fd, parm = "x1", type = "simultaneous", nsim = 2500)
 ##' head(x1.sint)
 ##' \dontshow{options(op)}
 `confint.fderiv` <- function(object, parm, level = 0.95,
-                             type = c("confidence", "simultaneous"), nsim = 10000, ...) {
+                             type = c("confidence", "simultaneous"), nsim = 10000,
+                             ncores = 1L, ...) {
     ## Process arguments
     ## parm is one of the terms in object
     parm <- if(missing(parm)) {
@@ -86,7 +93,8 @@
     interval <- if (type == "confidence") {
         confidence(object, terms = parm, level = level)
     } else {
-        simultaneous(object, terms = parm, level = level, nsim = nsim)
+        simultaneous(object, terms = parm, level = level, nsim = nsim,
+                     ncores = ncores)
     }
 
     class(interval) <- c("confint.fderiv", "data.frame")
@@ -96,8 +104,8 @@
 }
 
 ##' @importFrom stats quantile vcov
-##' @importFrom mvtnorm rmvnorm
-`simultaneous` <- function(x, terms, level, nsim) {
+##' @importFrom mvnfast rmvn
+`simultaneous` <- function(x, terms, level, nsim, ncores) {
     ## wrapper the computes each interval
     `simInt` <- function(x, Vb, bu, level, nsim) {
         Xi <- x[["Xi"]]           # derivative Lp, zeroed except for this term
@@ -115,7 +123,7 @@
     ## bayesian covar matrix, possibly accounting for estimating smooth pars
     Vb <- vcov(x[["model"]], unconditional = x$unconditional)
     ## simulate un-biased deviations given bayesian covar matrix
-    buDiff <- rmvnorm(n = nsim, mean = rep(0, nrow(Vb)), sigma = Vb)
+    buDiff <- rmvn(n = nsim, mu = rep(0, nrow(Vb)), sigma = Vb, ncores = ncores)
     ## apply wrapper to compute simultaneous interval critical value and
     ## corresponding simultaneous interval for each term
     res <- lapply(x[["derivatives"]][terms], FUN = simInt,
@@ -156,15 +164,32 @@
 ##' Calculates point-wise confidence or simultaneous intervals for the smooth terms of a fitted GAM.
 ##'
 ##' @param object an object of class `"gam"` or `"gamm"`.
-##' @param parm which parameters (smooth terms) are to be given intervals as a vector of terms. If missing, all parameters are considered, although this is not currently implemented.
-##' @param level numeric, `0 < level < 1`; the confidence level of the point-wise or simultaneous interval. The default is `0.95` for a 95\% interval.
-##' @param newdata data frame; containing new values of the covariates used in the model fit. The selected smooth(s) wil be evaluated at the supplied values.
+##' @param parm which parameters (smooth terms) are to be given intervals as a
+##'   vector of terms. If missing, all parameters are considered, although this
+##'   is not currently implemented.
+##' @param level numeric, `0 < level < 1`; the confidence level of the point-wise
+##'   or simultaneous interval. The default is `0.95` for a 95% interval.
+##' @param newdata data frame; containing new values of the covariates used in
+##'   the model fit. The selected smooth(s) wil be evaluated at the supplied
+##'   values.
 ##' @param n numeric; the number of points to evaluate smooths at.
-##' @param type character; the type of interval to compute. One of `"confidence"` for point-wise intervals, or `"simultaneous"` for simultaneous intervals.
-##' @param nsim integer; the number of simulations used in computing the simultaneous intervals.
+##' @param type character; the type of interval to compute. One of `"confidence"`
+##'   for point-wise intervals, or `"simultaneous"` for simultaneous intervals.
+##' @param nsim integer; the number of simulations used in computing the
+##'   simultaneous intervals.
 ##' @param shift logical; should the constant term be add to the smooth?
-##' @param transform logical; should the smooth be evaluated on a transformed scale? For generalised models, this involves applying the inverse of the link function used to fit the model. Alternatively, the name of, or an actual, function can be supplied to transform the smooth and it's confidence interval.
-##' @param unconditional logical; if `TRUE` (and `freq == FALSE`) then the Bayesian smoothing parameter uncertainty corrected covariance matrix is returned, if available.
+##' @param transform logical; should the smooth be evaluated on a transformed
+##'   scale? For generalised models, this involves applying the inverse of the
+##'   link function used to fit the model. Alternatively, the name of, or an
+##'   actual, function can be supplied to transform the smooth and it's
+##'   confidence interval.
+##' @param unconditional logical; if `TRUE` (and `freq == FALSE`) then the
+##'   Bayesian smoothing parameter uncertainty corrected covariance matrix is
+##'   returned, if available.
+##' @param ncores number of cores for generating random variables from a
+##'   multivariate normal distribution. Passed to [mvnfast::rmvn()].
+##'   Parallelization will take place only if OpenMP is supported (but appears
+##'   to work on Windows with current `R`).
 ##' @param ... additional arguments for methods
 ##'
 ##' @return a data frame with components:
@@ -180,29 +205,38 @@
 ##' @importFrom stats family qnorm
 ##' @importFrom mgcv PredictMat
 ##' @importFrom stats quantile vcov setNames
-##' @importFrom mvtnorm rmvnorm
 ##' @importFrom dplyr bind_rows
 ##' @importFrom tibble add_column
+##' @importFrom mvnfast rmvn
 ##'
 ##' @export
 ##'
 ##' @examples
-##'suppressPackageStartupMessages(library("mgcv"))
+##' load_mgcv()
+##' \dontshow{
 ##' set.seed(2)
-##' dat <- gamSim(1, n = 400, dist = "normal", scale = 2)
+##' op <- options(digits = 4, cli.unicode = FALSE)
+##' }
+##' dat <- gamSim(1, n = 500, dist = "normal", scale = 2)
 ##' mod <- gam(y ~ s(x0) + s(x1) + s(x2) + s(x3), data = dat, method = "REML")
-##' ##'
+##'
 ##' ## point-wise interval
 ##' ci <- confint(mod, parm = "x1", type = "confidence")
-##' head(ci)
-##' ##'
+##' ci
+##'
 ##' ## simultaneous interval for smooth term of x1
+##' \dontshow{
 ##' set.seed(42)
+##' }
 ##' si <- confint(mod, parm = "x1", type = "simultaneous", nsim = 100)
-##' head(si)
+##' si
+##' \dontshow{
+##' options(op)
+##' }
 `confint.gam` <- function(object, parm, level = 0.95, newdata = NULL, n = 200,
                           type = c("confidence", "simultaneous"), nsim = 10000,
                           shift = FALSE, transform = FALSE, unconditional = FALSE,
+                          ncores = 1,
                           ...) {
     parm <- add_s(parm)
     ## parm <- select_smooth(object, parm) # select_terms(object, parm)
@@ -249,7 +283,8 @@
         V <- get_vcov(object, unconditional = unconditional)
 
         ## simulate un-biased deviations given bayesian covar matrix
-        buDiff <- rmvnorm(n = nsim, mean = rep(0, nrow(V)), sigma = V)
+        buDiff <- rmvn(n = nsim, mu = rep(0, nrow(V)), sigma = V,
+                       ncores = ncores)
     }
     ## list to hold results
     out <- vector("list", length = length(uS)) # list for results
@@ -278,7 +313,9 @@
         ## need VCOV for simultaneous intervals
         V <- get_vcov(object, unconditional = unconditional)
         ## simulate un-biased deviations given bayesian covar matrix
-        buDiff <- rmvnorm(n = nsim, mean = rep(0, nrow(V)), sigma = V)
+        buDiff <- rmvn(n = nsim, mu = rep(0, nrow(V)), sigma = V,
+                       ncores = ncores)
+          #rmvnorm(n = nsim, mean = rep(0, nrow(V)), sigma = V)
         ## loop over smooths
         for (i in seq_along(out)) {
             ## evaluate smooth
