@@ -33,7 +33,7 @@
 ##' @export
 ##'
 ##' @examples
-##' suppressPackageStartupMessages(library("mgcv"))
+##' load_mgcv()
 ##' \dontshow{
 ##' set.seed(2)
 ##' op <- options(cli.unicode = FALSE, digits = 6)
@@ -110,7 +110,7 @@
 }
 
 ## Random effect smooth
-##' @importFrom tibble add_column
+##' @importFrom tibble add_column tibble
 `evaluate_re_smooth` <- function(object, model = NULL, newdata = NULL,
                                  unconditional = FALSE) {
     ## is this a by smooth
@@ -152,10 +152,10 @@
         para_seq <- seq(from = start, to = end, by = 1L)
         coefs <- coef(model)[para_seq]
         se <- diag(vcov(model, unconditional = unconditional))[para_seq]
-        evaluated[[i]] <- data_frame(smooth = rep(smooth_labels[i], length(coefs)),
-                                     ..var  = levs,
-                                     est = coefs,
-                                     se = se)
+        evaluated[[i]] <- tibble(smooth = rep(smooth_labels[i], length(coefs)),
+                                 ..var  = levs,
+                                 est = coefs,
+                                 se = se)
     }
 
     evaluated <- do.call("rbind", evaluated)
@@ -536,14 +536,17 @@
 ##' @rdname evaluate_smooth
 ##'
 ##' @importFrom stats delete.response
-##' @importFrom tibble data_frame add_column
+##' @importFrom tibble as_tibble add_column
+##' @importFrom rlang .data
+##' @importFrom dplyr mutate bind_cols bind_rows
 ##'
 ##' @export
 `evaluate_parametric_term.gam` <- function(object, term, unconditional = FALSE,
                                            ...) {
     tt <- object$pterms       # get parametric terms
     tt <- delete.response(tt) # remove response so easier to work with
-    vars <- labels(tt)        # names of all parametric terms
+    vars <- parametric_terms(object)
+    mgcv_names <- names(vars) # this is how mgcv refers to the terms
 
     if (length(term) > 1L) {
         term <- term[1L]
@@ -558,32 +561,46 @@
     mf <- model.frame(object)  # data used to fit model
     is_fac <- is.factor(mf[[term]]) # is term a factor?
 
+    ## match the specific term, with term names mgcv actually uses
+    ## for example in a model with multiple linear predictors, terms in
+    ## nth linear predictor (for n > 1) get appended .{n-1}  
+    ind <- match(term, vars)
+
+    ## take the actual mgcv version of the names for the `terms` argument
     evaluated <- as.data.frame(predict(object, newdata = mf, type = 'terms',
-                                       terms = term, se = TRUE,
+                                       terms = mgcv_names[ind], se = TRUE,
                                        unconditional = unconditional))
     evaluated <- setNames(evaluated, c("partial", "se"))
+    evaluated <- as_tibble(evaluated)
 
     if (is_fac) {
         levs <- levels(mf[, term])
         newd <- setNames(data.frame(fac = factor(levs, levels = levs)), "value")
         spl <- lapply(split(evaluated, mf[, term]), `[`, i = 1, j = )
-        evaluated <- cbind(term = term, type = ifelse(is_fac, "factor", "numeric"),
-                      newd, do.call("rbind", spl))
+        evaluated <- bind_rows(spl)
+        nr <- NROW(evaluated)
+        evaluated <- bind_cols(term = rep(term, nr),
+                               type = rep(ifelse(is_fac, "factor", "numeric"), nr),
+                               newd, evaluated)
     } else {
-        evaluated <- cbind(term = term, type = ifelse(is_fac, "factor", "numeric"),
-                           value = mf[, term], evaluated)
+        nr <- NROW(evaluated)
+        evaluated <- bind_cols(term = rep(term, nr),
+                               type = rep(ifelse(is_fac, "factor", "numeric"), nr),
+                               value = mf[[term]],
+                               evaluated)
     }
 
     ## add confidence interval
-    evaluated[["upper"]] <- evaluated[["partial"]] + (2 * evaluated[["se"]])
-    evaluated[["lower"]] <- evaluated[["partial"]] - (2 * evaluated[["se"]])
+    evaluated <- mutate(evaluated,
+                        upper = .data$partial + (2 * .data$se),
+                        lower = .data$partial - (2 * .data$se))
 
-    class(evaluated) <- c("evaluated_parametric_term", "data.frame")
+    class(evaluated) <- c("evaluated_parametric_term", class(evaluated))
     evaluated                           # return
 }
 
 ## loop over smooths and predict
-##' @importFrom tibble data_frame
+##' @importFrom tibble tibble
 `spline_values` <- function(smooth, newdata, model, unconditional,
                             overall_uncertainty = TRUE, term) {
     X <- PredictMat(smooth, newdata)   # prediction matrix
@@ -623,18 +640,18 @@
     ## Return
     out <- if (d == 1L) {
                if (is_fs_smooth(smooth)) {
-                   data_frame(smooth = rep(label, nrow(X)),
-                              x = newdata[, 1L], f = newdata[, 2L],
-                              est = fit, se = se.fit)
+                   tibble(smooth = rep(label, nrow(X)),
+                          x = newdata[, 1L], f = newdata[, 2L],
+                          est = fit, se = se.fit)
                } else {
-                   data_frame(smooth = rep(label, nrow(X)),
-                              x = newdata[, 1L],
-                              est = fit, se = se.fit)
+                   tibble(smooth = rep(label, nrow(X)),
+                          x = newdata[, 1L],
+                          est = fit, se = se.fit)
                }
            } else {
-               data_frame(smooth = rep(label, nrow(X)),
-                          x1 = newdata[, 1L], x2 = newdata[, 2L],
-                          est = fit, se = se.fit)
+               tibble(smooth = rep(label, nrow(X)),
+                      x1 = newdata[, 1L], x2 = newdata[, 2L],
+                      est = fit, se = se.fit)
            }
     out
 }
